import { FinalRequestOptions } from "../internal/request-options.mjs";
import { type Knock } from "../client.mjs";
import { APIPromise } from "./api-promise.mjs";
import { type APIResponseProps } from "../internal/parse.mjs";
export type PageRequestOptions = Pick<FinalRequestOptions, 'query' | 'headers' | 'body' | 'path' | 'method'>;
export declare abstract class AbstractPage<Item> implements AsyncIterable<Item> {
    #private;
    protected options: FinalRequestOptions;
    protected response: Response;
    protected body: unknown;
    constructor(client: Knock, response: Response, body: unknown, options: FinalRequestOptions);
    abstract nextPageRequestOptions(): PageRequestOptions | null;
    abstract getPaginatedItems(): Item[];
    hasNextPage(): boolean;
    getNextPage(): Promise<this>;
    iterPages(): AsyncGenerator<this>;
    [Symbol.asyncIterator](): AsyncGenerator<Item>;
}
/**
 * This subclass of Promise will resolve to an instantiated Page once the request completes.
 *
 * It also implements AsyncIterable to allow auto-paginating iteration on an unawaited list call, eg:
 *
 *    for await (const item of client.items.list()) {
 *      console.log(item)
 *    }
 */
export declare class PagePromise<PageClass extends AbstractPage<Item>, Item = ReturnType<PageClass['getPaginatedItems']>[number]> extends APIPromise<PageClass> implements AsyncIterable<Item> {
    constructor(client: Knock, request: Promise<APIResponseProps>, Page: new (...args: ConstructorParameters<typeof AbstractPage>) => PageClass);
    /**
     * Allow auto-paginating iteration on an unawaited list call, eg:
     *
     *    for await (const item of client.items.list()) {
     *      console.log(item)
     *    }
     */
    [Symbol.asyncIterator](): AsyncGenerator<Item>;
}
export interface EntriesCursorResponse<Item> {
    entries: Array<Item>;
    page_info: EntriesCursorResponse.PageInfo;
}
export declare namespace EntriesCursorResponse {
    interface PageInfo {
        after?: string;
    }
}
export interface EntriesCursorParams {
    after?: string;
    before?: string;
    page_size?: number;
}
export declare class EntriesCursor<Item> extends AbstractPage<Item> implements EntriesCursorResponse<Item> {
    entries: Array<Item>;
    page_info: EntriesCursorResponse.PageInfo;
    constructor(client: Knock, response: Response, body: EntriesCursorResponse<Item>, options: FinalRequestOptions);
    getPaginatedItems(): Item[];
    nextPageRequestOptions(): PageRequestOptions | null;
}
export interface ItemsCursorResponse<Item> {
    items: Array<Item>;
    page_info: ItemsCursorResponse.PageInfo;
}
export declare namespace ItemsCursorResponse {
    interface PageInfo {
        after?: string;
    }
}
export interface ItemsCursorParams {
    after?: string;
    before?: string;
    page_size?: number;
}
export declare class ItemsCursor<Item> extends AbstractPage<Item> implements ItemsCursorResponse<Item> {
    items: Array<Item>;
    page_info: ItemsCursorResponse.PageInfo;
    constructor(client: Knock, response: Response, body: ItemsCursorResponse<Item>, options: FinalRequestOptions);
    getPaginatedItems(): Item[];
    nextPageRequestOptions(): PageRequestOptions | null;
}
export interface SlackChannelsCursorResponse<Item> {
    next_cursor: string;
    slack_channels: Array<Item>;
}
export interface SlackChannelsCursorParams {
    'query_options.cursor'?: string;
    'query_options.limit'?: number;
}
export declare class SlackChannelsCursor<Item> extends AbstractPage<Item> implements SlackChannelsCursorResponse<Item> {
    next_cursor: string;
    slack_channels: Array<Item>;
    constructor(client: Knock, response: Response, body: SlackChannelsCursorResponse<Item>, options: FinalRequestOptions);
    getPaginatedItems(): Item[];
    nextPageRequestOptions(): PageRequestOptions | null;
}
export interface MsTeamsPaginationResponse<Item> {
    skip_token: string;
    ms_teams_teams: Array<Item>;
}
export interface MsTeamsPaginationParams {
    'query_options.$skiptoken'?: string;
    'query_options.$top'?: number;
}
export declare class MsTeamsPagination<Item> extends AbstractPage<Item> implements MsTeamsPaginationResponse<Item> {
    skip_token: string;
    ms_teams_teams: Array<Item>;
    constructor(client: Knock, response: Response, body: MsTeamsPaginationResponse<Item>, options: FinalRequestOptions);
    getPaginatedItems(): Item[];
    nextPageRequestOptions(): PageRequestOptions | null;
}
//# sourceMappingURL=pagination.d.mts.map