// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
var _AbstractPage_client;
import { __classPrivateFieldGet, __classPrivateFieldSet } from "../internal/tslib.mjs";
import { KnockError } from "./error.mjs";
import { defaultParseResponse } from "../internal/parse.mjs";
import { APIPromise } from "./api-promise.mjs";
import { maybeObj } from "../internal/utils/values.mjs";
export class AbstractPage {
    constructor(client, response, body, options) {
        _AbstractPage_client.set(this, void 0);
        __classPrivateFieldSet(this, _AbstractPage_client, client, "f");
        this.options = options;
        this.response = response;
        this.body = body;
    }
    hasNextPage() {
        const items = this.getPaginatedItems();
        if (!items.length)
            return false;
        return this.nextPageRequestOptions() != null;
    }
    async getNextPage() {
        const nextOptions = this.nextPageRequestOptions();
        if (!nextOptions) {
            throw new KnockError('No next page expected; please check `.hasNextPage()` before calling `.getNextPage()`.');
        }
        return await __classPrivateFieldGet(this, _AbstractPage_client, "f").requestAPIList(this.constructor, nextOptions);
    }
    async *iterPages() {
        let page = this;
        yield page;
        while (page.hasNextPage()) {
            page = await page.getNextPage();
            yield page;
        }
    }
    async *[(_AbstractPage_client = new WeakMap(), Symbol.asyncIterator)]() {
        for await (const page of this.iterPages()) {
            for (const item of page.getPaginatedItems()) {
                yield item;
            }
        }
    }
}
/**
 * This subclass of Promise will resolve to an instantiated Page once the request completes.
 *
 * It also implements AsyncIterable to allow auto-paginating iteration on an unawaited list call, eg:
 *
 *    for await (const item of client.items.list()) {
 *      console.log(item)
 *    }
 */
export class PagePromise extends APIPromise {
    constructor(client, request, Page) {
        super(client, request, async (client, props) => new Page(client, props.response, await defaultParseResponse(client, props), props.options));
    }
    /**
     * Allow auto-paginating iteration on an unawaited list call, eg:
     *
     *    for await (const item of client.items.list()) {
     *      console.log(item)
     *    }
     */
    async *[Symbol.asyncIterator]() {
        const page = await this;
        for await (const item of page) {
            yield item;
        }
    }
}
export class EntriesCursor extends AbstractPage {
    constructor(client, response, body, options) {
        super(client, response, body, options);
        this.entries = body.entries || [];
        this.page_info = body.page_info || {};
    }
    getPaginatedItems() {
        return this.entries ?? [];
    }
    nextPageRequestOptions() {
        const cursor = this.page_info?.after;
        if (!cursor) {
            return null;
        }
        return {
            ...this.options,
            query: {
                ...maybeObj(this.options.query),
                after: cursor,
            },
        };
    }
}
export class ItemsCursor extends AbstractPage {
    constructor(client, response, body, options) {
        super(client, response, body, options);
        this.items = body.items || [];
        this.page_info = body.page_info || {};
    }
    getPaginatedItems() {
        return this.items ?? [];
    }
    nextPageRequestOptions() {
        const cursor = this.page_info?.after;
        if (!cursor) {
            return null;
        }
        return {
            ...this.options,
            query: {
                ...maybeObj(this.options.query),
                after: cursor,
            },
        };
    }
}
export class SlackChannelsCursor extends AbstractPage {
    constructor(client, response, body, options) {
        super(client, response, body, options);
        this.next_cursor = body.next_cursor || '';
        this.slack_channels = body.slack_channels || [];
    }
    getPaginatedItems() {
        return this.slack_channels ?? [];
    }
    nextPageRequestOptions() {
        const cursor = this.next_cursor;
        if (!cursor) {
            return null;
        }
        return {
            ...this.options,
            query: {
                ...maybeObj(this.options.query),
                'query_options.cursor': cursor,
            },
        };
    }
}
export class MsTeamsPagination extends AbstractPage {
    constructor(client, response, body, options) {
        super(client, response, body, options);
        this.skip_token = body.skip_token || '';
        this.ms_teams_teams = body.ms_teams_teams || [];
    }
    getPaginatedItems() {
        return this.ms_teams_teams ?? [];
    }
    nextPageRequestOptions() {
        const cursor = this.skip_token;
        if (!cursor) {
            return null;
        }
        return {
            ...this.options,
            query: {
                ...maybeObj(this.options.query),
                'query_options.$skiptoken': cursor,
            },
        };
    }
}
//# sourceMappingURL=pagination.mjs.map