"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Audiences = void 0;
const resource_1 = require("../core/resource.js");
const path_1 = require("../internal/utils/path.js");
class Audiences extends resource_1.APIResource {
    /**
     * Adds one or more members to the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.addMembers('key', {
     *   members: [
     *     {
     *       tenant: 'ingen_isla_nublar',
     *       user: { id: 'dr_sattler' },
     *     },
     *   ],
     * });
     * ```
     */
    addMembers(key, body, options) {
        return this._client.post((0, path_1.path) `/v1/audiences/${key}/members`, { body, ...options });
    }
    /**
     * Returns a paginated list of members for the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.listMembers('key');
     * ```
     */
    listMembers(key, options) {
        return this._client.get((0, path_1.path) `/v1/audiences/${key}/members`, options);
    }
    /**
     * Removes one or more members from the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.removeMembers(
     *   'key',
     *   { members: [{ user: {} }] },
     * );
     * ```
     */
    removeMembers(key, body, options) {
        return this._client.delete((0, path_1.path) `/v1/audiences/${key}/members`, { body, ...options });
    }
}
exports.Audiences = Audiences;
//# sourceMappingURL=audiences.js.map