// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../core/resource.mjs";
import { path } from "../internal/utils/path.mjs";
export class Audiences extends APIResource {
    /**
     * Adds one or more members to the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.addMembers('key', {
     *   members: [
     *     {
     *       tenant: 'ingen_isla_nublar',
     *       user: { id: 'dr_sattler' },
     *     },
     *   ],
     * });
     * ```
     */
    addMembers(key, body, options) {
        return this._client.post(path `/v1/audiences/${key}/members`, { body, ...options });
    }
    /**
     * Returns a paginated list of members for the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.listMembers('key');
     * ```
     */
    listMembers(key, options) {
        return this._client.get(path `/v1/audiences/${key}/members`, options);
    }
    /**
     * Removes one or more members from the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.removeMembers(
     *   'key',
     *   { members: [{ user: {} }] },
     * );
     * ```
     */
    removeMembers(key, body, options) {
        return this._client.delete(path `/v1/audiences/${key}/members`, { body, ...options });
    }
}
//# sourceMappingURL=audiences.mjs.map