"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bulk = void 0;
const resource_1 = require("../../core/resource.js");
const path_1 = require("../../internal/utils/path.js");
class Bulk extends resource_1.APIResource {
    /**
     * Bulk update the status of messages for a specific channel. The channel is
     * specified by the `channel_id` parameter. The action to perform is specified by
     * the `action` parameter, where the action is a status change action (e.g.
     * `archive`, `unarchive`).
     *
     * @example
     * ```ts
     * const bulkOperation =
     *   await client.channels.bulk.updateMessageStatus(
     *     '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     *     'seen',
     *     {
     *       archived: 'include',
     *       delivery_status: 'delivered',
     *       engagement_status: 'seen',
     *       has_tenant: true,
     *       newer_than: '2024-01-01T00:00:00Z',
     *       older_than: '2024-01-01T00:00:00Z',
     *       recipient_ids: ['recipient1', 'recipient2'],
     *       tenants: ['tenant1', 'tenant2'],
     *       trigger_data: '{"key":"value"}',
     *       workflows: ['workflow1', 'workflow2'],
     *     },
     *   );
     * ```
     */
    updateMessageStatus(channelID, action, body, options) {
        return this._client.post((0, path_1.path) `/v1/channels/${channelID}/messages/bulk/${action}`, { body, ...options });
    }
}
exports.Bulk = Bulk;
//# sourceMappingURL=bulk.js.map