// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
export class Batch extends APIResource {
    /**
     * Marks the given messages as archived. Archived messages are hidden from the
     * default message list in the feed but can still be accessed and unarchived later.
     *
     * @example
     * ```ts
     * const messages = await client.messages.batch.archive({
     *   message_ids: [
     *     '2w3YUpTTOxuDvZFji8OMsKrG176',
     *     '2w3YVRbPXMIh8Zq6oBFcVDA5xes',
     *   ],
     * });
     * ```
     */
    archive(body, options) {
        return this._client.post('/v1/messages/batch/archived', { body, ...options });
    }
    /**
     * Get the contents of multiple messages in a single request.
     *
     * @example
     * ```ts
     * const response = await client.messages.batch.getContent({
     *   message_ids: ['string'],
     * });
     * ```
     */
    getContent(query, options) {
        return this._client.get('/v1/messages/batch/content', { query, ...options });
    }
    /**
     * Marks the given messages as interacted with by the user. This can include any
     * user action on the message, with optional metadata about the specific
     * interaction. Cannot include more than 5 key-value pairs, must not contain nested
     * data. Read more about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const messages =
     *   await client.messages.batch.markAsInteracted({
     *     message_ids: ['1jNaXzB2RZX3LY8wVQnfCKyPnv7'],
     *     metadata: { key: 'value' },
     *   });
     * ```
     */
    markAsInteracted(body, options) {
        return this._client.post('/v1/messages/batch/interacted', { body, ...options });
    }
    /**
     * Marks the given messages as `read`. Read more about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const messages = await client.messages.batch.markAsRead({
     *   message_ids: [
     *     '2w3YUpTTOxuDvZFji8OMsKrG176',
     *     '2w3YVRbPXMIh8Zq6oBFcVDA5xes',
     *   ],
     * });
     * ```
     */
    markAsRead(body, options) {
        return this._client.post('/v1/messages/batch/read', { body, ...options });
    }
    /**
     * Marks the given messages as `seen`. This indicates that the user has viewed the
     * message in their feed or inbox. Read more about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const messages = await client.messages.batch.markAsSeen({
     *   message_ids: [
     *     '2w3YUpTTOxuDvZFji8OMsKrG176',
     *     '2w3YVRbPXMIh8Zq6oBFcVDA5xes',
     *   ],
     * });
     * ```
     */
    markAsSeen(body, options) {
        return this._client.post('/v1/messages/batch/seen', { body, ...options });
    }
    /**
     * Marks the given messages as `unread`. This reverses the `read` state. Read more
     * about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const messages = await client.messages.batch.markAsUnread({
     *   message_ids: [
     *     '2w3YUpTTOxuDvZFji8OMsKrG176',
     *     '2w3YVRbPXMIh8Zq6oBFcVDA5xes',
     *   ],
     * });
     * ```
     */
    markAsUnread(body, options) {
        return this._client.post('/v1/messages/batch/unread', { body, ...options });
    }
    /**
     * Marks the given messages as `unseen`. This reverses the `seen` state. Read more
     * about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const messages = await client.messages.batch.markAsUnseen({
     *   message_ids: [
     *     '2w3YUpTTOxuDvZFji8OMsKrG176',
     *     '2w3YVRbPXMIh8Zq6oBFcVDA5xes',
     *   ],
     * });
     * ```
     */
    markAsUnseen(body, options) {
        return this._client.post('/v1/messages/batch/unseen', { body, ...options });
    }
    /**
     * Marks the given messages as unarchived. This reverses the `archived` state.
     * Archived messages are hidden from the default message list in the feed but can
     * still be accessed and unarchived later.
     *
     * @example
     * ```ts
     * const messages = await client.messages.batch.unarchive({
     *   message_ids: [
     *     '2w3YUpTTOxuDvZFji8OMsKrG176',
     *     '2w3YVRbPXMIh8Zq6oBFcVDA5xes',
     *   ],
     * });
     * ```
     */
    unarchive(body, options) {
        return this._client.post('/v1/messages/batch/unarchived', { body, ...options });
    }
}
//# sourceMappingURL=batch.mjs.map