// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
import * as BatchAPI from "./batch.mjs";
import { Batch, } from "./batch.mjs";
import { ItemsCursor } from "../../core/pagination.mjs";
import { path } from "../../internal/utils/path.mjs";
export class Messages extends APIResource {
    constructor() {
        super(...arguments);
        this.batch = new BatchAPI.Batch(this._client);
    }
    /**
     * Returns a paginated list of messages for the current environment.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const message of client.messages.list()) {
     *   // ...
     * }
     * ```
     */
    list(query = {}, options) {
        return this._client.getAPIList('/v1/messages', (ItemsCursor), { query, ...options });
    }
    /**
     * Archives a message for the user. Archived messages are hidden from the default
     * message list in the feed but can still be accessed and unarchived later.
     *
     * @example
     * ```ts
     * const message = await client.messages.archive(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    archive(messageID, options) {
        return this._client.put(path `/v1/messages/${messageID}/archived`, options);
    }
    /**
     * Retrieves a specific message by its ID.
     *
     * @example
     * ```ts
     * const message = await client.messages.get(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    get(messageID, options) {
        return this._client.get(path `/v1/messages/${messageID}`, options);
    }
    /**
     * Returns the fully rendered contents of a message, where the response depends on
     * which channel the message was sent through.
     *
     * @example
     * ```ts
     * const response = await client.messages.getContent(
     *   'message_id',
     * );
     * ```
     */
    getContent(messageID, options) {
        return this._client.get(path `/v1/messages/${messageID}/content`, options);
    }
    /**
     * Returns a paginated list of activities for the specified message.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const activity of client.messages.listActivities(
     *   'message_id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listActivities(messageID, query = {}, options) {
        return this._client.getAPIList(path `/v1/messages/${messageID}/activities`, (ItemsCursor), {
            query,
            ...options,
        });
    }
    /**
     * Returns a paginated list of delivery logs for the specified message.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const messageDeliveryLog of client.messages.listDeliveryLogs(
     *   'message_id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listDeliveryLogs(messageID, query = {}, options) {
        return this._client.getAPIList(path `/v1/messages/${messageID}/delivery_logs`, (ItemsCursor), { query, ...options });
    }
    /**
     * Returns a paginated list of events for the specified message.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const messageEvent of client.messages.listEvents(
     *   'message_id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listEvents(messageID, query = {}, options) {
        return this._client.getAPIList(path `/v1/messages/${messageID}/events`, (ItemsCursor), {
            query,
            ...options,
        });
    }
    /**
     * Marks a message as `interacted` with by the user. This can include any user
     * action on the message, with optional metadata about the specific interaction.
     * Cannot include more than 5 key-value pairs, must not contain nested data. Read
     * more about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const message = await client.messages.markAsInteracted(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    markAsInteracted(messageID, body = {}, options) {
        return this._client.put(path `/v1/messages/${messageID}/interacted`, { body, ...options });
    }
    /**
     * Marks a message as `read`. This indicates that the user has read the message
     * content. Read more about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const message = await client.messages.markAsRead(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    markAsRead(messageID, options) {
        return this._client.put(path `/v1/messages/${messageID}/read`, options);
    }
    /**
     * Marks a message as `seen`. This indicates that the user has viewed the message
     * in their feed or inbox. Read more about message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const message = await client.messages.markAsSeen(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    markAsSeen(messageID, options) {
        return this._client.put(path `/v1/messages/${messageID}/seen`, options);
    }
    /**
     * Marks a message as `unread`. This reverses the `read` state. Read more about
     * message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const message = await client.messages.markAsUnread(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    markAsUnread(messageID, options) {
        return this._client.delete(path `/v1/messages/${messageID}/read`, options);
    }
    /**
     * Marks a message as `unseen`. This reverses the `seen` state. Read more about
     * message engagement statuses
     * [here](/send-notifications/message-statuses#engagement-status).
     *
     * @example
     * ```ts
     * const message = await client.messages.markAsUnseen(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    markAsUnseen(messageID, options) {
        return this._client.delete(path `/v1/messages/${messageID}/seen`, options);
    }
    /**
     * Removes a message from the archived state, making it visible in the default
     * message list in the feed again.
     *
     * @example
     * ```ts
     * const message = await client.messages.unarchive(
     *   '1jNaXzB2RZX3LY8wVQnfCKyPnv7',
     * );
     * ```
     */
    unarchive(messageID, options) {
        return this._client.delete(path `/v1/messages/${messageID}/archived`, options);
    }
}
Messages.Batch = Batch;
//# sourceMappingURL=messages.mjs.map