"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bulk = void 0;
const resource_1 = require("../../core/resource.js");
const path_1 = require("../../internal/utils/path.js");
class Bulk extends resource_1.APIResource {
    /**
     * Bulk deletes objects from the specified collection.
     *
     * @example
     * ```ts
     * const bulkOperation = await client.objects.bulk.delete(
     *   'collection',
     *   { object_ids: ['obj_123', 'obj_456', 'obj_789'] },
     * );
     * ```
     */
    delete(collection, body, options) {
        return this._client.post((0, path_1.path) `/v1/objects/${collection}/bulk/delete`, { body, ...options });
    }
    /**
     * Add subscriptions for all objects in a single collection. If a subscription for
     * an object in the collection already exists, it will be updated. This endpoint
     * also handles
     * [inline identifications](/managing-recipients/identifying-recipients#inline-identifying-recipients)
     * for the `recipient` field.
     *
     * @example
     * ```ts
     * const bulkOperation =
     *   await client.objects.bulk.addSubscriptions('projects', {
     *     subscriptions: [
     *       {
     *         id: 'project-1',
     *         properties: null,
     *         recipients: [{ id: 'user_1' }],
     *       },
     *     ],
     *   });
     * ```
     */
    addSubscriptions(collection, body, options) {
        return this._client.post((0, path_1.path) `/v1/objects/${collection}/bulk/subscriptions/add`, { body, ...options });
    }
    /**
     * Bulk sets up to 1,000 objects at a time in the specified collection.
     *
     * @example
     * ```ts
     * const bulkOperation = await client.objects.bulk.set(
     *   'collection',
     *   {
     *     objects: [
     *       {
     *         id: 'project_1',
     *         name: {
     *           '0': 'M',
     *           '1': 'y',
     *           '2': ' ',
     *           '3': 'p',
     *           '4': 'r',
     *           '5': 'o',
     *           '6': 'j',
     *           '7': 'e',
     *           '8': 'c',
     *           '9': 't',
     *         },
     *       },
     *     ],
     *   },
     * );
     * ```
     */
    set(collection, body, options) {
        return this._client.post((0, path_1.path) `/v1/objects/${collection}/bulk/set`, { body, ...options });
    }
}
exports.Bulk = Bulk;
//# sourceMappingURL=bulk.js.map