"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Objects = void 0;
const tslib_1 = require("../../internal/tslib.js");
const resource_1 = require("../../core/resource.js");
const BulkAPI = tslib_1.__importStar(require("./bulk.js"));
const bulk_1 = require("./bulk.js");
const pagination_1 = require("../../core/pagination.js");
const path_1 = require("../../internal/utils/path.js");
class Objects extends resource_1.APIResource {
    constructor() {
        super(...arguments);
        this.bulk = new BulkAPI.Bulk(this._client);
    }
    /**
     * Returns a paginated list of objects from the specified collection. Optionally
     * includes preference data for the objects.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const object of client.objects.list(
     *   'collection',
     * )) {
     *   // ...
     * }
     * ```
     */
    list(collection, query = {}, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/objects/${collection}`, (pagination_1.EntriesCursor), {
            query,
            ...options,
        });
    }
    /**
     * Permanently removes an object from the specified collection. This operation
     * cannot be undone.
     *
     * @example
     * ```ts
     * const object = await client.objects.delete(
     *   'collection',
     *   'id',
     * );
     * ```
     */
    delete(collection, id, options) {
        return this._client.delete((0, path_1.path) `/v1/objects/${collection}/${id}`, options);
    }
    /**
     * Add subscriptions for an object. If a subscription already exists, it will be
     * updated. This endpoint also handles
     * [inline identifications](/managing-recipients/identifying-recipients#inline-identifying-recipients)
     * for the `recipient`.
     *
     * @example
     * ```ts
     * const subscriptions = await client.objects.addSubscriptions(
     *   'collection',
     *   'object_id',
     *   {
     *     recipients: ['user_1', 'user_2'],
     *     properties: { key: 'value' },
     *   },
     * );
     * ```
     */
    addSubscriptions(collection, objectID, body, options) {
        return this._client.post((0, path_1.path) `/v1/objects/${collection}/${objectID}/subscriptions`, { body, ...options });
    }
    /**
     * Delete subscriptions for the specified recipients from an object. Returns the
     * list of deleted subscriptions.
     *
     * @example
     * ```ts
     * const subscriptions =
     *   await client.objects.deleteSubscriptions(
     *     'collection',
     *     'object_id',
     *     { recipients: ['user_123'] },
     *   );
     * ```
     */
    deleteSubscriptions(collection, objectID, body, options) {
        return this._client.delete((0, path_1.path) `/v1/objects/${collection}/${objectID}/subscriptions`, {
            body,
            ...options,
        });
    }
    /**
     * Retrieves a specific object by its ID from the specified collection. Returns the
     * object with all its properties.
     *
     * @example
     * ```ts
     * const object = await client.objects.get('collection', 'id');
     * ```
     */
    get(collection, id, options) {
        return this._client.get((0, path_1.path) `/v1/objects/${collection}/${id}`, options);
    }
    /**
     * Returns the channel data for the specified object and channel.
     *
     * @example
     * ```ts
     * const channelData = await client.objects.getChannelData(
     *   'collection',
     *   'object_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * );
     * ```
     */
    getChannelData(collection, objectID, channelID, options) {
        return this._client.get((0, path_1.path) `/v1/objects/${collection}/${objectID}/channel_data/${channelID}`, options);
    }
    /**
     * Returns the preference set for the specified object and preference set `id`.
     *
     * @example
     * ```ts
     * const preferenceSet = await client.objects.getPreferences(
     *   'collection',
     *   'object_id',
     *   'default',
     * );
     * ```
     */
    getPreferences(collection, objectID, id, options) {
        return this._client.get((0, path_1.path) `/v1/objects/${collection}/${objectID}/preferences/${id}`, options);
    }
    /**
     * Returns a paginated list of messages for a specific object in the given
     * collection. Allows filtering by message status and provides various sorting
     * options.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const message of client.objects.listMessages(
     *   'projects',
     *   'project-123',
     * )) {
     *   // ...
     * }
     * ```
     */
    listMessages(collection, id, query = {}, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/objects/${collection}/${id}/messages`, (pagination_1.ItemsCursor), { query, ...options });
    }
    /**
     * Returns a paginated list of preference sets for the specified object.
     *
     * @example
     * ```ts
     * const preferenceSets = await client.objects.listPreferences(
     *   'collection',
     *   'object_id',
     * );
     * ```
     */
    listPreferences(collection, objectID, options) {
        return this._client.get((0, path_1.path) `/v1/objects/${collection}/${objectID}/preferences`, options);
    }
    /**
     * Returns a paginated list of schedules for an object.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const schedule of client.objects.listSchedules(
     *   'collection',
     *   'id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listSchedules(collection, id, query = {}, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/objects/${collection}/${id}/schedules`, (pagination_1.EntriesCursor), { query, ...options });
    }
    /**
     * List subscriptions for an object. Either list the recipients that subscribe to
     * the provided object, or list the objects that the provided object is subscribed
     * to. Determined by the `mode` query parameter.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const subscription of client.objects.listSubscriptions(
     *   'collection',
     *   'object_id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listSubscriptions(collection, objectID, query = {}, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/objects/${collection}/${objectID}/subscriptions`, (pagination_1.EntriesCursor), { query, ...options });
    }
    /**
     * Creates a new object or updates an existing one in the specified collection.
     * This operation is used to identify objects with their properties, as well as
     * optional preferences and channel data.
     *
     * @example
     * ```ts
     * const object = await client.objects.set(
     *   'collection',
     *   'id',
     *   {
     *     channel_data: {
     *       '97c5837d-c65c-4d54-aa39-080eeb81c69d': {
     *         tokens: ['push_token_123'],
     *       },
     *     },
     *     locale: 'en-US',
     *     preferences: {
     *       default: {
     *         channel_types: { email: true },
     *         workflows: {
     *           'dinosaurs-loose': {
     *             channel_types: { email: true },
     *           },
     *         },
     *       },
     *     },
     *     timezone: 'America/New_York',
     *   },
     * );
     * ```
     */
    set(collection, id, body, options) {
        return this._client.put((0, path_1.path) `/v1/objects/${collection}/${id}`, { body, ...options });
    }
    /**
     * Sets the channel data for the specified object and channel. If no object exists
     * in the current environment for the given `collection` and `object_id`, Knock
     * will create the object as part of this request.
     *
     * @example
     * ```ts
     * const channelData = await client.objects.setChannelData(
     *   'collection',
     *   'object_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     *   { data: { tokens: ['push_token_1'] } },
     * );
     * ```
     */
    setChannelData(collection, objectID, channelID, body, options) {
        return this._client.put((0, path_1.path) `/v1/objects/${collection}/${objectID}/channel_data/${channelID}`, {
            body,
            ...options,
        });
    }
    /**
     * Sets preferences within the given preference set. This is a destructive
     * operation and will replace any existing preferences with the preferences given.
     * If no object exists in the current environment for the given `:collection` and
     * `:object_id`, Knock will create the object as part of this request. The
     * preference set `:id` can be either `default` or a `tenant.id`. Learn more about
     * [per-tenant preferences](/preferences/tenant-preferences).
     *
     * @example
     * ```ts
     * const preferenceSet = await client.objects.setPreferences(
     *   'collection',
     *   'object_id',
     *   'default',
     *   {
     *     categories: {
     *       marketing: false,
     *       transactional: { channel_types: { email: false } },
     *     },
     *     channel_types: { email: true },
     *     workflows: {
     *       'dinosaurs-loose': {
     *         channel_types: { email: false },
     *       },
     *     },
     *   },
     * );
     * ```
     */
    setPreferences(collection, objectID, id, body, options) {
        return this._client.put((0, path_1.path) `/v1/objects/${collection}/${objectID}/preferences/${id}`, {
            body,
            ...options,
        });
    }
    /**
     * Unsets the channel data for the specified object and channel.
     *
     * @example
     * ```ts
     * const response = await client.objects.unsetChannelData(
     *   'collection',
     *   'object_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * );
     * ```
     */
    unsetChannelData(collection, objectID, channelID, options) {
        return this._client.delete((0, path_1.path) `/v1/objects/${collection}/${objectID}/channel_data/${channelID}`, options);
    }
}
exports.Objects = Objects;
Objects.Bulk = bulk_1.Bulk;
//# sourceMappingURL=objects.js.map