"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.MsTeams = void 0;
const resource_1 = require("../../core/resource.js");
const pagination_1 = require("../../core/pagination.js");
const path_1 = require("../../internal/utils/path.js");
class MsTeams extends resource_1.APIResource {
    /**
     * Check if a connection to Microsoft Teams has been authorized for a given
     * Microsoft Teams tenant object.
     *
     * @example
     * ```ts
     * const response = await client.providers.msTeams.checkAuth(
     *   'channel_id',
     *   { ms_teams_tenant_object: 'ms_teams_tenant_object' },
     * );
     * ```
     */
    checkAuth(channelID, query, options) {
        return this._client.get((0, path_1.path) `/v1/providers/ms-teams/${channelID}/auth_check`, { query, ...options });
    }
    /**
     * List the Microsoft Teams channels within a team. By default, archived and
     * private channels are excluded from the results.
     *
     * @example
     * ```ts
     * const response =
     *   await client.providers.msTeams.listChannels(
     *     'channel_id',
     *     {
     *       ms_teams_tenant_object: 'ms_teams_tenant_object',
     *       team_id: 'team_id',
     *     },
     *   );
     * ```
     */
    listChannels(channelID, query, options) {
        return this._client.get((0, path_1.path) `/v1/providers/ms-teams/${channelID}/channels`, { query, ...options });
    }
    /**
     * Get a list of teams belonging to the Microsoft Entra tenant. By default,
     * archived and private channels are excluded from the results.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const msTeamListTeamsResponse of client.providers.msTeams.listTeams(
     *   'channel_id',
     *   { ms_teams_tenant_object: 'ms_teams_tenant_object' },
     * )) {
     *   // ...
     * }
     * ```
     */
    listTeams(channelID, query, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/providers/ms-teams/${channelID}/teams`, (pagination_1.MsTeamsPagination), { query, ...options });
    }
    /**
     * Remove a Microsoft Entra tenant ID from a Microsoft Teams tenant object.
     *
     * @example
     * ```ts
     * const response =
     *   await client.providers.msTeams.revokeAccess(
     *     'channel_id',
     *     { ms_teams_tenant_object: 'ms_teams_tenant_object' },
     *   );
     * ```
     */
    revokeAccess(channelID, params, options) {
        const { ms_teams_tenant_object } = params;
        return this._client.put((0, path_1.path) `/v1/providers/ms-teams/${channelID}/revoke_access`, {
            query: { ms_teams_tenant_object },
            ...options,
        });
    }
}
exports.MsTeams = MsTeams;
//# sourceMappingURL=ms-teams.js.map