"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Slack = void 0;
const resource_1 = require("../../core/resource.js");
const pagination_1 = require("../../core/pagination.js");
const path_1 = require("../../internal/utils/path.js");
class Slack extends resource_1.APIResource {
    /**
     * Check if a Slack channel is authenticated.
     *
     * @example
     * ```ts
     * const response = await client.providers.slack.checkAuth(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * );
     * ```
     */
    checkAuth(channelID, query, options) {
        return this._client.get((0, path_1.path) `/v1/providers/slack/${channelID}/auth_check`, { query, ...options });
    }
    /**
     * List Slack channels for a Slack workspace.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const slackListChannelsResponse of client.providers.slack.listChannels(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * )) {
     *   // ...
     * }
     * ```
     */
    listChannels(channelID, query, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/providers/slack/${channelID}/channels`, (pagination_1.SlackChannelsCursor), { query, ...options });
    }
    /**
     * Revoke access for a Slack channel.
     *
     * @example
     * ```ts
     * const response = await client.providers.slack.revokeAccess(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * );
     * ```
     */
    revokeAccess(channelID, params, options) {
        const { access_token_object } = params;
        return this._client.put((0, path_1.path) `/v1/providers/slack/${channelID}/revoke_access`, {
            query: { access_token_object },
            ...options,
        });
    }
}
exports.Slack = Slack;
//# sourceMappingURL=slack.js.map