// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
import { SlackChannelsCursor } from "../../core/pagination.mjs";
import { path } from "../../internal/utils/path.mjs";
export class Slack extends APIResource {
    /**
     * Check if a Slack channel is authenticated.
     *
     * @example
     * ```ts
     * const response = await client.providers.slack.checkAuth(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * );
     * ```
     */
    checkAuth(channelID, query, options) {
        return this._client.get(path `/v1/providers/slack/${channelID}/auth_check`, { query, ...options });
    }
    /**
     * List Slack channels for a Slack workspace.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const slackListChannelsResponse of client.providers.slack.listChannels(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * )) {
     *   // ...
     * }
     * ```
     */
    listChannels(channelID, query, options) {
        return this._client.getAPIList(path `/v1/providers/slack/${channelID}/channels`, (SlackChannelsCursor), { query, ...options });
    }
    /**
     * Revoke access for a Slack channel.
     *
     * @example
     * ```ts
     * const response = await client.providers.slack.revokeAccess(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * );
     * ```
     */
    revokeAccess(channelID, params, options) {
        const { access_token_object } = params;
        return this._client.put(path `/v1/providers/slack/${channelID}/revoke_access`, {
            query: { access_token_object },
            ...options,
        });
    }
}
//# sourceMappingURL=slack.mjs.map