// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
import * as BulkAPI from "./bulk.mjs";
import { Bulk } from "./bulk.mjs";
import { EntriesCursor } from "../../core/pagination.mjs";
export class Schedules extends APIResource {
    constructor() {
        super(...arguments);
        this.bulk = new BulkAPI.Bulk(this._client);
    }
    /**
     * Creates one or more schedules for a workflow with the specified recipients,
     * timing, and data. Schedules can be one-time or recurring. This endpoint also
     * handles
     * [inline identifications](/managing-recipients/identifying-recipients#inline-identifying-recipients)
     * for the `actor`, `recipient`, and `tenant` fields.
     *
     * @example
     * ```ts
     * const schedules = await client.schedules.create({
     *   recipients: ['user_123'],
     *   workflow: 'comment-created',
     *   data: { key: 'value' },
     *   repeats: [
     *     {
     *       __typename: 'ScheduleRepeat',
     *       day_of_month: null,
     *       days: [
     *         'mon',
     *         'tue',
     *         'wed',
     *         'thu',
     *         'fri',
     *         'sat',
     *         'sun',
     *       ],
     *       frequency: 'daily',
     *       hours: null,
     *       interval: 1,
     *       minutes: null,
     *     },
     *   ],
     *   tenant: 'acme_corp',
     * });
     * ```
     */
    create(body, options) {
        return this._client.post('/v1/schedules', { body, ...options });
    }
    /**
     * Updates one or more existing schedules with new timing, data, or other
     * properties. All specified schedule IDs will be updated with the same values.
     * This endpoint also handles
     * [inline identifications](/managing-recipients/identifying-recipients#inline-identifying-recipients)
     * for the `actor`, `recipient`, and `tenant` fields.
     *
     * @example
     * ```ts
     * const schedules = await client.schedules.update({
     *   schedule_ids: ['123e4567-e89b-12d3-a456-426614174000'],
     *   data: { key: 'value' },
     *   repeats: [
     *     {
     *       __typename: 'ScheduleRepeat',
     *       day_of_month: null,
     *       days: [
     *         'mon',
     *         'tue',
     *         'wed',
     *         'thu',
     *         'fri',
     *         'sat',
     *         'sun',
     *       ],
     *       frequency: 'daily',
     *       hours: null,
     *       interval: 1,
     *       minutes: null,
     *     },
     *   ],
     *   tenant: 'acme_corp',
     * });
     * ```
     */
    update(body, options) {
        return this._client.put('/v1/schedules', { body, ...options });
    }
    /**
     * Returns a paginated list of schedules for the current environment, filtered by
     * workflow and optionally by recipients and tenant.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const schedule of client.schedules.list({
     *   workflow: 'workflow',
     * })) {
     *   // ...
     * }
     * ```
     */
    list(query, options) {
        return this._client.getAPIList('/v1/schedules', (EntriesCursor), { query, ...options });
    }
    /**
     * Permanently deletes one or more schedules identified by the provided schedule
     * IDs. This operation cannot be undone.
     *
     * @example
     * ```ts
     * const schedules = await client.schedules.delete({
     *   schedule_ids: ['123e4567-e89b-12d3-a456-426614174000'],
     * });
     * ```
     */
    delete(body, options) {
        return this._client.delete('/v1/schedules', { body, ...options });
    }
}
Schedules.Bulk = Bulk;
//# sourceMappingURL=schedules.mjs.map