import { APIResource } from "../../core/resource.js";
import * as BulkOperationsAPI from "../bulk-operations.js";
import * as TenantsAPI from "./tenants.js";
import { APIPromise } from "../../core/api-promise.js";
import { RequestOptions } from "../../internal/request-options.js";
export declare class Bulk extends APIResource {
    /**
     * Delete multiple tenants in a single operation. This operation cannot be undone.
     *
     * @example
     * ```ts
     * const bulkOperation = await client.tenants.bulk.delete({
     *   tenant_ids: ['string'],
     * });
     * ```
     */
    delete(params: BulkDeleteParams, options?: RequestOptions): APIPromise<BulkOperationsAPI.BulkOperation>;
    /**
     * Set or update up to 1,000 tenants in a single operation.
     *
     * @example
     * ```ts
     * const bulkOperation = await client.tenants.bulk.set({
     *   tenants: ['[object Object]'],
     * });
     * ```
     */
    set(body: BulkSetParams, options?: RequestOptions): APIPromise<BulkOperationsAPI.BulkOperation>;
}
export interface BulkDeleteParams {
    /**
     * The IDs of the tenants to delete.
     */
    tenant_ids: Array<string>;
}
export interface BulkSetParams {
    /**
     * The tenants to be upserted.
     */
    tenants: Array<TenantsAPI.InlineTenantRequest>;
}
export declare namespace Bulk {
    export { type BulkDeleteParams as BulkDeleteParams, type BulkSetParams as BulkSetParams };
}
//# sourceMappingURL=bulk.d.ts.map