"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tenants = void 0;
const tslib_1 = require("../../internal/tslib.js");
const resource_1 = require("../../core/resource.js");
const BulkAPI = tslib_1.__importStar(require("./bulk.js"));
const bulk_1 = require("./bulk.js");
const pagination_1 = require("../../core/pagination.js");
const path_1 = require("../../internal/utils/path.js");
class Tenants extends resource_1.APIResource {
    constructor() {
        super(...arguments);
        this.bulk = new BulkAPI.Bulk(this._client);
    }
    /**
     * List tenants for the current environment.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const tenant of client.tenants.list()) {
     *   // ...
     * }
     * ```
     */
    list(query = {}, options) {
        return this._client.getAPIList('/v1/tenants', (pagination_1.EntriesCursor), { query, ...options });
    }
    /**
     * Delete a tenant and all associated data. This operation cannot be undone.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.delete('id');
     * ```
     */
    delete(id, options) {
        return this._client.delete((0, path_1.path) `/v1/tenants/${id}`, options);
    }
    /**
     * Get a tenant by ID.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.get('id');
     * ```
     */
    get(id, options) {
        return this._client.get((0, path_1.path) `/v1/tenants/${id}`, options);
    }
    /**
     * Sets a tenant within an environment, performing an upsert operation. Any
     * existing properties will be merged with the incoming properties.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.set('id', {
     *   settings: {
     *     branding: {
     *       icon_url:
     *         'https://example.com/trex_silhouette_icon.png',
     *       logo_url: 'https://example.com/amber_fossil_logo.png',
     *       primary_color: '#DF1A22',
     *       primary_color_contrast: '#FFDE00',
     *     },
     *   },
     * });
     * ```
     */
    set(id, body, options) {
        return this._client.put((0, path_1.path) `/v1/tenants/${id}`, { body, ...options });
    }
}
exports.Tenants = Tenants;
Tenants.Bulk = bulk_1.Bulk;
//# sourceMappingURL=tenants.js.map