// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
import * as BulkAPI from "./bulk.mjs";
import { Bulk } from "./bulk.mjs";
import { EntriesCursor } from "../../core/pagination.mjs";
import { path } from "../../internal/utils/path.mjs";
export class Tenants extends APIResource {
    constructor() {
        super(...arguments);
        this.bulk = new BulkAPI.Bulk(this._client);
    }
    /**
     * List tenants for the current environment.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const tenant of client.tenants.list()) {
     *   // ...
     * }
     * ```
     */
    list(query = {}, options) {
        return this._client.getAPIList('/v1/tenants', (EntriesCursor), { query, ...options });
    }
    /**
     * Delete a tenant and all associated data. This operation cannot be undone.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.delete('id');
     * ```
     */
    delete(id, options) {
        return this._client.delete(path `/v1/tenants/${id}`, options);
    }
    /**
     * Get a tenant by ID.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.get('id');
     * ```
     */
    get(id, options) {
        return this._client.get(path `/v1/tenants/${id}`, options);
    }
    /**
     * Sets a tenant within an environment, performing an upsert operation. Any
     * existing properties will be merged with the incoming properties.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.set('id', {
     *   settings: {
     *     branding: {
     *       icon_url:
     *         'https://example.com/trex_silhouette_icon.png',
     *       logo_url: 'https://example.com/amber_fossil_logo.png',
     *       primary_color: '#DF1A22',
     *       primary_color_contrast: '#FFDE00',
     *     },
     *   },
     * });
     * ```
     */
    set(id, body, options) {
        return this._client.put(path `/v1/tenants/${id}`, { body, ...options });
    }
}
Tenants.Bulk = Bulk;
//# sourceMappingURL=tenants.mjs.map