"use strict";
// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Feeds = void 0;
const resource_1 = require("../../core/resource.js");
const pagination_1 = require("../../core/pagination.js");
const path_1 = require("../../internal/utils/path.js");
class Feeds extends resource_1.APIResource {
    /**
     * Returns the feed settings for a user.
     *
     * @example
     * ```ts
     * const response = await client.users.feeds.getSettings(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * );
     * ```
     */
    getSettings(userID, id, options) {
        return this._client.get((0, path_1.path) `/v1/users/${userID}/feeds/${id}/settings`, options);
    }
    /**
     * Returns a paginated list of feed items for a user, including metadata about the
     * feed.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const feedListItemsResponse of client.users.feeds.listItems(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * )) {
     *   // ...
     * }
     * ```
     */
    listItems(userID, id, query = {}, options) {
        return this._client.getAPIList((0, path_1.path) `/v1/users/${userID}/feeds/${id}`, (pagination_1.EntriesCursor), { query, ...options });
    }
}
exports.Feeds = Feeds;
//# sourceMappingURL=feeds.js.map