// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
import { EntriesCursor } from "../../core/pagination.mjs";
import { path } from "../../internal/utils/path.mjs";
export class Feeds extends APIResource {
    /**
     * Returns the feed settings for a user.
     *
     * @example
     * ```ts
     * const response = await client.users.feeds.getSettings(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * );
     * ```
     */
    getSettings(userID, id, options) {
        return this._client.get(path `/v1/users/${userID}/feeds/${id}/settings`, options);
    }
    /**
     * Returns a paginated list of feed items for a user, including metadata about the
     * feed.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const feedListItemsResponse of client.users.feeds.listItems(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * )) {
     *   // ...
     * }
     * ```
     */
    listItems(userID, id, query = {}, options) {
        return this._client.getAPIList(path `/v1/users/${userID}/feeds/${id}`, (EntriesCursor), { query, ...options });
    }
}
//# sourceMappingURL=feeds.mjs.map