// File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.
import { APIResource } from "../../core/resource.mjs";
import * as BulkAPI from "./bulk.mjs";
import { Bulk } from "./bulk.mjs";
import * as FeedsAPI from "./feeds.mjs";
import { Feeds, } from "./feeds.mjs";
import * as GuidesAPI from "./guides.mjs";
import { Guides, } from "./guides.mjs";
import { EntriesCursor, ItemsCursor, } from "../../core/pagination.mjs";
import { path } from "../../internal/utils/path.mjs";
export class Users extends APIResource {
    constructor() {
        super(...arguments);
        this.feeds = new FeedsAPI.Feeds(this._client);
        this.guides = new GuidesAPI.Guides(this._client);
        this.bulk = new BulkAPI.Bulk(this._client);
    }
    /**
     * Create or update a user with the provided identification data. When you identify
     * an existing user, the system merges the properties you specific with what is
     * currently set on the user, updating only the fields included in your requests.
     *
     * @example
     * ```ts
     * const user = await client.users.update('user_id', {
     *   channel_data: {
     *     '97c5837d-c65c-4d54-aa39-080eeb81c69d': {
     *       tokens: ['push_token_123'],
     *     },
     *   },
     *   email: 'ian.malcolm@chaos.theory',
     *   name: 'Dr. Ian Malcolm',
     *   preferences: {
     *     default: {
     *       channel_types: { email: true },
     *       workflows: {
     *         'dinosaurs-loose': {
     *           channel_types: { email: true },
     *         },
     *       },
     *     },
     *   },
     *   timezone: 'America/New_York',
     * });
     * ```
     */
    update(userID, body, options) {
        return this._client.put(path `/v1/users/${userID}`, { body, ...options });
    }
    /**
     * Retrieve a paginated list of users in the environment. Defaults to 50 users per
     * page.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const user of client.users.list()) {
     *   // ...
     * }
     * ```
     */
    list(query = {}, options) {
        return this._client.getAPIList('/v1/users', (EntriesCursor), { query, ...options });
    }
    /**
     * Permanently delete a user and all associated data.
     *
     * @example
     * ```ts
     * const user = await client.users.delete('user_id');
     * ```
     */
    delete(userID, options) {
        return this._client.delete(path `/v1/users/${userID}`, options);
    }
    /**
     * Retrieve a specific user by their ID.
     *
     * @example
     * ```ts
     * const user = await client.users.get('user_id');
     * ```
     */
    get(userID, options) {
        return this._client.get(path `/v1/users/${userID}`, options);
    }
    /**
     * Retrieves the channel data for a specific user and channel ID.
     *
     * @example
     * ```ts
     * const channelData = await client.users.getChannelData(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * );
     * ```
     */
    getChannelData(userID, channelID, options) {
        return this._client.get(path `/v1/users/${userID}/channel_data/${channelID}`, options);
    }
    /**
     * Retrieves a specific preference set for a user identified by the preference set
     * ID.
     *
     * @example
     * ```ts
     * const preferenceSet = await client.users.getPreferences(
     *   'user_id',
     *   'default',
     * );
     * ```
     */
    getPreferences(userID, id, query = {}, options) {
        return this._client.get(path `/v1/users/${userID}/preferences/${id}`, { query, ...options });
    }
    /**
     * Returns a paginated list of messages for a specific user. Allows filtering by
     * message status and provides various sorting options. Messages outside the
     * account's retention window will not be included in the results.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const message of client.users.listMessages(
     *   'user-123',
     * )) {
     *   // ...
     * }
     * ```
     */
    listMessages(userID, query = {}, options) {
        return this._client.getAPIList(path `/v1/users/${userID}/messages`, (ItemsCursor), {
            query,
            ...options,
        });
    }
    /**
     * Retrieves a list of all preference sets for a specific user.
     *
     * @example
     * ```ts
     * const preferenceSets = await client.users.listPreferences(
     *   'user_id',
     * );
     * ```
     */
    listPreferences(userID, options) {
        return this._client.get(path `/v1/users/${userID}/preferences`, options);
    }
    /**
     * Returns a paginated list of schedules for a specific user, in descending order.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const schedule of client.users.listSchedules(
     *   'user_id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listSchedules(userID, query = {}, options) {
        return this._client.getAPIList(path `/v1/users/${userID}/schedules`, (EntriesCursor), { query, ...options });
    }
    /**
     * Retrieves a paginated list of subscriptions for a specific user, in descending
     * order.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const subscription of client.users.listSubscriptions(
     *   'user_id',
     * )) {
     *   // ...
     * }
     * ```
     */
    listSubscriptions(userID, query = {}, options) {
        return this._client.getAPIList(path `/v1/users/${userID}/subscriptions`, (EntriesCursor), { query, ...options });
    }
    /**
     * Merge two users together, where the user specified with the `from_user_id` param
     * will be merged into the user specified by `user_id`.
     *
     * @example
     * ```ts
     * const user = await client.users.merge('user_id', {
     *   from_user_id: 'user_1',
     * });
     * ```
     */
    merge(userID, body, options) {
        return this._client.post(path `/v1/users/${userID}/merge`, { body, ...options });
    }
    /**
     * Updates or creates channel data for a specific user and channel ID. If no user
     * exists in the current environment for the given `user_id`, Knock will create the
     * user entry as part of this request.
     *
     * @example
     * ```ts
     * const channelData = await client.users.setChannelData(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     *   { data: { tokens: ['push_token_1'] } },
     * );
     * ```
     */
    setChannelData(userID, channelID, body, options) {
        return this._client.put(path `/v1/users/${userID}/channel_data/${channelID}`, { body, ...options });
    }
    /**
     * Updates a complete preference set for a user. This is a destructive operation
     * that will replace the existing preference set for the user.
     *
     * @example
     * ```ts
     * const preferenceSet = await client.users.setPreferences(
     *   'user_id',
     *   'default',
     *   {
     *     categories: {
     *       marketing: false,
     *       transactional: { channel_types: { email: false } },
     *     },
     *     channel_types: { email: true },
     *     workflows: {
     *       'dinosaurs-loose': {
     *         channel_types: { email: false },
     *       },
     *     },
     *   },
     * );
     * ```
     */
    setPreferences(userID, id, body, options) {
        return this._client.put(path `/v1/users/${userID}/preferences/${id}`, { body, ...options });
    }
    /**
     * Deletes channel data for a specific user and channel ID.
     *
     * @example
     * ```ts
     * const response = await client.users.unsetChannelData(
     *   'user_id',
     *   '182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e',
     * );
     * ```
     */
    unsetChannelData(userID, channelID, options) {
        return this._client.delete(path `/v1/users/${userID}/channel_data/${channelID}`, options);
    }
}
Users.Feeds = Feeds;
Users.Guides = Guides;
Users.Bulk = Bulk;
//# sourceMappingURL=users.mjs.map